/************************************
 
    NEKO FOR WIN95
    C++ Conversion

 *************************************/

#include <windows.h>
#include <math.h>
#include "Neko95.hpp"
#include "NekoSettings.hpp"
#include "desktop.h"
#include "resource.h"
#include "NekoCommon.h"

/*****************************************************/

CNeko::CNeko( HINSTANCE hInstance, HWND hWnd/*=NULL*/ )
{
    //initialise member variables
    m_hwndOldInstance = FindWindow( szNekoClassName, szNekoWindowTitle );
    m_hInstance = hInstance;
    m_hwndDefault = GetDesktopHandle();
    m_hiconLastDrawn = NULL;
    m_hDC = NULL;
    m_iDX = m_iDY = 0;

    SetWindow( hWnd );
    SetState( STOP );
    

    //set Neko default position
    //(off to bottom right)
    //m_iX = m_rectBounds.right; m_iY = m_rectBounds.bottom;
    //(center)
    m_iX = (m_rectBounds.right-m_rectBounds.left) / 2; m_iY = (m_rectBounds.bottom-m_rectBounds.top) / 2;

    m_iOldX = m_iOldToX = m_iToX = m_iX;
    m_iOldY = m_iOldToY = m_iToY = m_iY;


    //set the default settings up
    m_dwSpeed = 16;
    m_dwIdleSpace = 6;
    strcpy( m_lpstrLibname, "" );

    //load 'em from the registry
    LoadConfiguration();
}

CNeko::~CNeko()
{
    Erase();
    DeleteIcons();
    if( m_hDC ) ReleaseDC( m_hwndCurrent, m_hDC );
}

BOOL CNeko::SetWindow( HWND hWnd/*=NULL*/ )
{
    if( hWnd == NULL ) hWnd = m_hwndDefault;
    m_hwndCurrent = hWnd;
    RecalculateWindowBounds();

    return TRUE;
}

BOOL CNeko::HasMoved()
{
    return( (m_iX == m_iOldX) && (m_iY == m_iOldY) );
}

void CNeko::LoadConfiguration()
{
    
    CNekoSettings NekoSettings( szNekoRegKey );

    NekoSettings.GetBool( szNekoSpeedKey, &m_bChaseRandom );
    NekoSettings.GetInt( szNekoSenseKey,  &m_dwChaseItem );
    NekoSettings.GetInt( szNekoSpeedKey,  &m_dwIndependence );

    NekoSettings.GetInt( szNekoSpeedKey,  &m_dwSpeed );
    NekoSettings.GetInt( szNekoSenseKey,  &m_dwIdleSpace );
    NekoSettings.GetString( szNekoLibraryKey, m_lpstrLibname, MAX_PATH-1 );

    //(re)load the image library
    SetImages( m_lpstrLibname );
}

BOOL CNeko::MoveStart()
{
    return( !(( m_iOldToX >= m_iToX-(int)m_dwIdleSpace ) && 
              ( m_iOldToX <= m_iToX+(int)m_dwIdleSpace ) &&
              ( m_iOldToY >= m_iToY-(int)m_dwIdleSpace ) && 
              ( m_iOldToY <= m_iToY+(int)m_dwIdleSpace )));
}

void CNeko::CalcDirection()
{
   #define PI_PER8         ((double)3.1415926535/(double)8)
   double SinPiPer8Times3 = sin(PI_PER8 * (double)3);
   double SinPiPer8 = sin(PI_PER8);

    States NewState;
    double LargeX, LargeY, Length, SinTheta;

    if( (m_iDX == 0) && (m_iDY == 0) )
        NewState = STOP;
    else 
    {
        LargeX = (double)m_iDX;
        LargeY = (double)(-m_iDY);
        Length = sqrt(LargeX * LargeX + LargeY * LargeY);
        SinTheta = LargeY / Length;

        if( m_iDX > 0 )
        {
            if( SinTheta > g_dSinPiPer8Times3 )
                NewState = U_MOVE;
            else 
                if( (SinTheta <= g_dSinPiPer8Times3 ) && ( SinTheta > g_dSinPiPer8 ) ) 
                    NewState = UR_MOVE;
                else 
                    if( (SinTheta <= g_dSinPiPer8) && (SinTheta > -(g_dSinPiPer8) ) ) 
                        NewState = R_MOVE;
                    else 
                        if( (SinTheta <= -(g_dSinPiPer8) ) && (SinTheta > -(g_dSinPiPer8Times3) ) ) 
                            NewState = DR_MOVE;
                        else 
                            NewState = D_MOVE;
        }
        else 
        {
            if( SinTheta > g_dSinPiPer8Times3 )
                NewState = U_MOVE;
            else 
                if( (SinTheta <= g_dSinPiPer8Times3) && (SinTheta > g_dSinPiPer8) ) 
                    NewState = UL_MOVE;
                else 
                    if( (SinTheta <= g_dSinPiPer8) && (SinTheta > -(g_dSinPiPer8) ) )
                        NewState = L_MOVE;
                    else 
                        if( (SinTheta <= -(g_dSinPiPer8)) && (SinTheta > -(g_dSinPiPer8Times3) ) ) 
                            NewState = DL_MOVE;
                        else 
                            NewState = D_MOVE;
        }
    }

    if( m_State != NewState ) SetState( NewState );

}


void CNeko::RunTowards( int iToX, int iToY )
{
    //calculate distance
    double LargeX, LargeY;
    double DoubleLength, Length;

    m_iOldToX = m_iToX;
    m_iOldToY = m_iToY;
    
    m_iToX = iToX - m_ptWindowPos.x;
    m_iToY = iToY - m_ptWindowPos.y;

    LargeX = (double)(m_iToX - m_iX - (int)m_uIconWidth / 2);
    LargeY = (double)(m_iToY - m_iY - (int)m_uIconHeight);

    DoubleLength = LargeX * LargeX + LargeY * LargeY;

    if (DoubleLength != (double)0) 
    {
            Length = sqrt(DoubleLength);
            if (Length <= (int)m_dwSpeed) 
            {
                m_iDX = (int)LargeX;
                m_iDY = (int)LargeY;
            } 
            else 
            {
                m_iDX = (int)(((int)m_dwSpeed * LargeX) / Length);
                m_iDY = (int)(((int)m_dwSpeed * LargeY) / Length);
            }
    } 
    else m_iDX = m_iDY = 0;


    //draw
    if ( m_State != SLEEP )
        Draw( m_hiconAnimation[m_State][m_uTickCount & 0x1] );
    else 
        Draw( m_hiconAnimation[m_State][(m_uTickCount>>2) & 0x1] );


    //increment animation counter
    if ( ++m_uTickCount >= MAX_TICK ) m_uTickCount = 0;
    
    if ( m_uTickCount%2 == 0 )
        if (m_uStateCount < MAX_TICK) m_uStateCount++;



    //change state
    switch( m_State ) 
    {
        case STOP:
            if( MoveStart() ) SetState( AWAKE ); 
            else
                if( m_uStateCount >= STOP_TIME ) 
                    if( (m_iDX < 0) && (m_iX <= 0)) SetState( L_CLAW ); 
                    else 
                        if( (m_iDX > 0) && (m_iX >= ( m_rectBounds.right - m_rectBounds.left ) - (int)m_uIconWidth ) ) SetState( R_CLAW ); 
                        else 
                            if( (m_iDY < 0) && (m_iY <= 0) ) SetState( U_CLAW ); 
                            else 
                                if( (m_iDY > 0) && (m_iY >= ( m_rectBounds.bottom - m_rectBounds.top ) - (int)m_uIconHeight ) ) SetState( D_CLAW );
                                else SetState( WASH );
            break;

        case WASH:
            if( MoveStart() ) SetState( AWAKE ); 
                else if( m_uStateCount >= WASH_TIME ) SetState( SCRATCH );
            break;

        case SCRATCH:
            if( MoveStart() ) SetState( AWAKE ); 
                else if (m_uStateCount >= SCRATCH_TIME ) SetState( YAWN );
            break;

        case YAWN:
            if( MoveStart() )  SetState( AWAKE );
                else if (m_uStateCount >= YAWN_TIME) SetState( SLEEP );
            break;
        
        case SLEEP:
            if( MoveStart() ) SetState( AWAKE );
            break;
        
        case AWAKE:
            if ( m_uStateCount >= AWAKE_TIME ) CalcDirection();
            break;
       
        case U_MOVE:
        case D_MOVE:
        case L_MOVE:
        case R_MOVE:
        case UL_MOVE:
        case UR_MOVE:
        case DL_MOVE:
        case DR_MOVE:
            m_iX += m_iDX;
            m_iY += m_iDY;
            CalcDirection();
            if ( IsWindowOver() && HasMoved() ) SetState(STOP);
            break;
        
        case U_CLAW:
        case D_CLAW:
        case L_CLAW:
        case R_CLAW:
            if( MoveStart() ) SetState( AWAKE );
                else if( m_uStateCount >= CLAW_TIME ) SetState( SCRATCH );
            break;

        default:
            //something bad has happened!
            MessageBeep( 0xFFFFFFFF );
            SetState( STOP );
            break;
    }
}

void CNeko::SetState( States state )
{
    //reset the animation counters
    m_uTickCount = 0;
    m_uStateCount = 0;

    //update the state
    m_State = state;
}

void CNeko::Draw( HICON frame )
{
    HDC dc;

    if( (m_iX != m_iOldX) || (m_iY != m_iOldY) || (frame != m_hiconLastDrawn) )
    {
        Erase();

        dc = GetDC( m_hwndCurrent );
        DrawIcon( dc, m_iX, m_iY, frame );
        ReleaseDC( m_hwndCurrent, dc );
    }

    m_iOldX = m_iX;
    m_iOldY = m_iY;

    m_hiconLastDrawn = frame;
}

void CNeko::Erase()
{
    RECT rect;

    rect.left = m_iOldX;
    rect.top = m_iOldY;
    rect.right = m_iOldX + m_uIconWidth;
    rect.bottom = m_iOldY + m_uIconHeight;

    //FIXME: Which is faster?
    //InvalidateRect( m_hwndCurrent, &rect, TRUE); UpdateWindow( m_hwndCurrent );
    RedrawWindow( m_hwndCurrent, &rect, NULL, RDW_ERASE|RDW_INVALIDATE|RDW_NOCHILDREN|RDW_UPDATENOW );
}

BOOL CNeko::IsWindowOver()
{
    BOOL ReturnValue = FALSE;

    if( m_iY <= 0 ) 
    {
        m_iY = 0;
        ReturnValue = TRUE;
    }
    else 
        if( m_iY >= (m_rectBounds.bottom-m_rectBounds.top) - (int)m_uIconHeight ) 
        {
            m_iY = (m_rectBounds.bottom-m_rectBounds.top) - (int)m_uIconHeight;
            ReturnValue = TRUE;
        }
    
    if( m_iX <= 0 ) 
    {
        m_iX = 0;
        ReturnValue = TRUE;
    }
    else
        if( m_iX >= (m_rectBounds.right-m_rectBounds.left) - (int)m_uIconWidth ) 
        {
            m_iX = (m_rectBounds.right-m_rectBounds.left) - (int)m_uIconWidth;
            ReturnValue = TRUE;
        }

    return ReturnValue;
}


void CNeko::UnloadPreviousInstance()
{
    PostMessage( m_hwndOldInstance, WM_QUIT, 0, 0 );
}

BOOL CNeko::SetImages( LPCSTR images/*=NULL*/ )
{
    char szEXEFilename[MAX_PATH+1];
    BOOL loadProblems = FALSE;

    if( ( images == NULL ) || ( strcmpi( images, "" ) == 0 ) )
    {
        //use the main Neko program for the images
        GetModuleFileName( NULL, szEXEFilename, MAX_PATH );
        images = szEXEFilename;
    }
    
    //free all of the current icons
    DestroyIcon( m_hiconAwake );
    DestroyIcon( m_hiconUp1 );
    DestroyIcon( m_hiconUp2 );
    DestroyIcon( m_hiconUpRight1 );
    DestroyIcon( m_hiconUpRight2 );
    DestroyIcon( m_hiconRight1 );
    DestroyIcon( m_hiconRight2 );
    DestroyIcon( m_hiconDownRight1 );
    DestroyIcon( m_hiconDownRight2 );
    DestroyIcon( m_hiconDown1 );
    DestroyIcon( m_hiconDown2 );
    DestroyIcon( m_hiconDownLeft1 );
    DestroyIcon( m_hiconDownLeft2 );
    DestroyIcon( m_hiconLeft1 );
    DestroyIcon( m_hiconLeft2 );
    DestroyIcon( m_hiconUpLeft1 );
    DestroyIcon( m_hiconUpLeft2 );
    DestroyIcon( m_hiconUpClaw1 );
    DestroyIcon( m_hiconUpClaw2 );
    DestroyIcon( m_hiconRightClaw1 );
    DestroyIcon( m_hiconRightClaw2 );
    DestroyIcon( m_hiconLeftClaw1 );
    DestroyIcon( m_hiconLeftClaw2 );
    DestroyIcon( m_hiconDownClaw1 );
    DestroyIcon( m_hiconDownClaw2 );
    DestroyIcon( m_hiconWash2 );
    DestroyIcon( m_hiconScratch1 );
    DestroyIcon( m_hiconScratch2 );
    DestroyIcon( m_hiconYawn2 );
    DestroyIcon( m_hiconYawn3 );
    DestroyIcon( m_hiconSleep1 );
    DestroyIcon( m_hiconSleep2 );
        
    //extract all the icons from the file
    m_hiconAwake =      ExtractIcon( m_hInstance, images, 0 );
    m_hiconUp1 =        ExtractIcon( m_hInstance, images, 1 );
    m_hiconUp2 =        ExtractIcon( m_hInstance, images, 2 );
    m_hiconUpRight1 =   ExtractIcon( m_hInstance, images, 3 );
    m_hiconUpRight2 =   ExtractIcon( m_hInstance, images, 4 );
    m_hiconRight1 =     ExtractIcon( m_hInstance, images, 5 );
    m_hiconRight2 =     ExtractIcon( m_hInstance, images, 6 );
    m_hiconDownRight1 = ExtractIcon( m_hInstance, images, 7 );
    m_hiconDownRight2 = ExtractIcon( m_hInstance, images, 8 );
    m_hiconDown1 =      ExtractIcon( m_hInstance, images, 9 );
    m_hiconDown2 =      ExtractIcon( m_hInstance, images, 10 );
    m_hiconDownLeft1 =  ExtractIcon( m_hInstance, images, 11 );
    m_hiconDownLeft2 =  ExtractIcon( m_hInstance, images, 12 );
    m_hiconLeft1 =      ExtractIcon( m_hInstance, images, 13 );
    m_hiconLeft2 =      ExtractIcon( m_hInstance, images, 14 );
    m_hiconUpLeft1 =    ExtractIcon( m_hInstance, images, 15 );
    m_hiconUpLeft2 =    ExtractIcon( m_hInstance, images, 16 );
    m_hiconUpClaw1 =    ExtractIcon( m_hInstance, images, 17 );
    m_hiconUpClaw2 =    ExtractIcon( m_hInstance, images, 18 );
    m_hiconRightClaw1 = ExtractIcon( m_hInstance, images, 19 );
    m_hiconRightClaw2 = ExtractIcon( m_hInstance, images, 20 );
    m_hiconLeftClaw1 =  ExtractIcon( m_hInstance, images, 21 );
    m_hiconLeftClaw2 =  ExtractIcon( m_hInstance, images, 22 );
    m_hiconDownClaw1 =  ExtractIcon( m_hInstance, images, 23 );
    m_hiconDownClaw2 =  ExtractIcon( m_hInstance, images, 24 );
    m_hiconWash2 =      ExtractIcon( m_hInstance, images, 25 );
    m_hiconScratch1 =   ExtractIcon( m_hInstance, images, 26 );
    m_hiconScratch2 =   ExtractIcon( m_hInstance, images, 27 );
    m_hiconYawn2 =      ExtractIcon( m_hInstance, images, 28 );
    m_hiconYawn3 =      ExtractIcon( m_hInstance, images, 29 );
    m_hiconSleep1 =     ExtractIcon( m_hInstance, images, 30 );
    m_hiconSleep2 =     ExtractIcon( m_hInstance, images, 31 );

    //check that there are enough icons in the file
    if( (UINT)m_hiconSleep1 <= 1)
    {
        MessageBox( NULL, "Could not find enough icons in the specified image library\nSome images may not be visible", "Neko", MB_ICONEXCLAMATION );
        loadProblems = TRUE;
    }

    //plug icons into animation table
    m_hiconAnimation[0 ][0] = m_hiconYawn2;       m_hiconAnimation[0 ][1] = m_hiconYawn2;
    m_hiconAnimation[1 ][0] = m_hiconWash2;       m_hiconAnimation[1 ][1] = m_hiconYawn2;
    m_hiconAnimation[2 ][0] = m_hiconScratch1;    m_hiconAnimation[2 ][1] = m_hiconScratch2;
    m_hiconAnimation[3 ][0] = m_hiconYawn3;       m_hiconAnimation[3 ][1] = m_hiconYawn3;
    m_hiconAnimation[4 ][0] = m_hiconSleep1;      m_hiconAnimation[4 ][1] = m_hiconSleep2;
    m_hiconAnimation[5 ][0] = m_hiconAwake;       m_hiconAnimation[5 ][1] = m_hiconAwake;
    m_hiconAnimation[6 ][0] = m_hiconUp1  ;       m_hiconAnimation[6 ][1] = m_hiconUp2;
    m_hiconAnimation[7 ][0] = m_hiconDown1;       m_hiconAnimation[7 ][1] = m_hiconDown2;
    m_hiconAnimation[8 ][0] = m_hiconLeft1;       m_hiconAnimation[8 ][1] = m_hiconLeft2;
    m_hiconAnimation[9 ][0] = m_hiconRight1;      m_hiconAnimation[9 ][1] = m_hiconRight2;
    m_hiconAnimation[10][0] = m_hiconUpLeft1;     m_hiconAnimation[10][1] = m_hiconUpLeft2;
    m_hiconAnimation[11][0] = m_hiconUpRight1;    m_hiconAnimation[11][1] = m_hiconUpRight2;
    m_hiconAnimation[12][0] = m_hiconDownLeft1;   m_hiconAnimation[12][1] = m_hiconDownLeft2;
    m_hiconAnimation[13][0] = m_hiconDownRight1;  m_hiconAnimation[13][1] = m_hiconDownRight2;
    m_hiconAnimation[14][0] = m_hiconUpClaw1;     m_hiconAnimation[14][1] = m_hiconUpClaw2;
    m_hiconAnimation[15][0] = m_hiconDownClaw1;   m_hiconAnimation[15][1] = m_hiconDownClaw2;
    m_hiconAnimation[16][0] = m_hiconLeftClaw1;   m_hiconAnimation[16][1] = m_hiconLeftClaw2;
    m_hiconAnimation[17][0] = m_hiconRightClaw1;  m_hiconAnimation[17][1] = m_hiconRightClaw2;


    //calculate the dimensions of the largest icon
    m_uIconWidth = 0;
    m_uIconHeight = 0;

    ICONINFO iconInfo;
    BITMAP bitmap;
    for( int index1 = 0; index1 < 18; index1++ )
        for( int index2 = 0; index2 < 2; index2++ )
        {
            GetIconInfo( m_hiconAnimation[index1][index2], &iconInfo );
            GetObject( (HBITMAP)iconInfo.hbmColor, sizeof(BITMAP), &bitmap );

            if( bitmap.bmWidth > (int)m_uIconWidth ) m_uIconWidth = bitmap.bmWidth;
            if( bitmap.bmHeight > (int)m_uIconHeight ) m_uIconHeight = bitmap.bmHeight;

            DeleteObject( iconInfo.hbmColor );
            DeleteObject( iconInfo.hbmMask );
        }

    //success?
    return loadProblems;
}

void CNeko::DeleteIcons()
{
    for( int index1 = 0; index1 < 18; index1++ )
        for( int index2 = 0; index2 < 2; index2++ )
            DeleteObject( m_hiconAnimation[index1][index2] );
}   

void CNeko::RecalculateWindowBounds()
{
    RECT rect;

    GetWindowRect( m_hwndCurrent, &rect );
    GetClientRect( m_hwndCurrent, &m_rectBounds );

    m_ptWindowPos.x = ( rect.right - m_rectBounds.right );
    m_ptWindowPos.y = ( rect.bottom - m_rectBounds.bottom );

    m_rectBounds.top += rect.top;
    m_rectBounds.left += rect.left;
    m_rectBounds.bottom += rect.top;
    m_rectBounds.right += rect.left;
}
