/************************************
 
    Neko's main header file

 *************************************/

#ifndef _NEKO95_HPP
#define _NEKO95_HPP

#include <windows.h>

//maths calculations
#define g_dSinPiPer8        0.3826834323651  // sin [pi/8]
#define g_dSinPiPer8Times3  0.9238795325113  // sin ( [pi/8] x 3 )

//misc. constants
#define MAX_TICK        9999 //Odd Only

//animation control constants
#define STOP_TIME      4
#define WASH_TIME      10
#define SCRATCH_TIME   4
#define YAWN_TIME      3
#define AWAKE_TIME     3
#define CLAW_TIME      10


//main Neko class definition
class CNeko {

public:
    CNeko( HINSTANCE hInstance, HWND hWnd = NULL  );
    ~CNeko();

    BOOL SetWindow( HWND hWnd = NULL );
    HWND GetWindow() { return m_hwndCurrent; };
    HWND GetDefaultWindow() { return m_hwndDefault; };

    BOOL SetImages( LPCSTR filename = NULL );
       
    void RunTowards( int iToX, int iToY );
    void LoadConfiguration();
    void RecalculateWindowBounds();

    BOOL IsPreviousInstance() { return (m_hwndOldInstance!=NULL); };
    void UnloadPreviousInstance();

    void GetPosition( int& x, int& y ) { x = m_iX; y = m_iY; };
    void SetPosition( int x, int y ) { m_iX = x; m_iY = y; };

    //attributes
    DWORD m_dwSpeed;
    DWORD m_dwIdleSpace;
    TCHAR m_lpstrLibname[MAX_PATH];

    DWORD m_dwIndependence;
    DWORD m_dwChaseItem;
    BOOL  m_bChaseRandom;

protected:
    enum States { STOP, WASH, SCRATCH, YAWN, SLEEP, AWAKE, U_MOVE, D_MOVE, L_MOVE, R_MOVE, UL_MOVE, UR_MOVE, DL_MOVE, DR_MOVE, U_CLAW, D_CLAW, L_CLAW, R_CLAW };

    BOOL HasMoved();
    void SetState( States state );

    RECT  m_rectBounds;
    POINT m_ptWindowPos;
    UINT  m_uIconWidth, m_uIconHeight;

    void Draw( HICON frame );
    void Erase();

private:

    void DeleteIcons();
    void CalcDirection();
    BOOL MoveStart();
    BOOL IsWindowOver();

    //position info
    int m_iX, m_iY;
    int m_iOldX, m_iOldY;
    int m_iDX, m_iDY;

    int m_iOldToX, m_iOldToY;
    int m_iToX, m_iToY;

    //state info
    UINT   m_uTickCount, m_uStateCount;
    States m_State;

    //windows stuff
    HWND      m_hwndOldInstance;
    HWND      m_hwndCurrent;
    HWND      m_hwndDefault;
    HINSTANCE m_hInstance;
    HDC       m_hDC;

    //icons
    HICON m_hiconAwake;
    HICON m_hiconWash2;
    HICON m_hiconScratch1,  m_hiconScratch2;
    HICON m_hiconYawn2,     m_hiconYawn3;
    HICON m_hiconSleep1,    m_hiconSleep2;
    HICON m_hiconUp1,       m_hiconUp2;
    HICON m_hiconDown1,     m_hiconDown2;
    HICON m_hiconLeft1,     m_hiconLeft2;
    HICON m_hiconRight1,    m_hiconRight2;
    HICON m_hiconUpLeft1,   m_hiconUpLeft2;
    HICON m_hiconUpRight1,  m_hiconUpRight2;
    HICON m_hiconDownLeft1, m_hiconDownLeft2;
    HICON m_hiconDownRight1,m_hiconDownRight2;
    HICON m_hiconUpClaw1,   m_hiconUpClaw2;
    HICON m_hiconDownClaw1, m_hiconDownClaw2;
    HICON m_hiconLeftClaw1, m_hiconLeftClaw2;
    HICON m_hiconRightClaw1,m_hiconRightClaw2;

    //most recently drawn icon
    HICON m_hiconLastDrawn;

    //location of animation frames
    HICON m_hiconAnimation[18][2];
};

#endif
