#include <windows.h>
#include <commctrl.h>
#include <commdlg.h>
#include "cfgres.h"
#include "NekoCommon.h"
#include "NekoSettings.hpp"

const char * szWindowTitle = "Configure Neko";

#define MIN_SPEED 2
#define MAX_SPEED 48
#define MIN_SENSE 1
#define MAX_SENSE 64

//global variables
HWND g_hwndNeko = NULL;
HWND g_hwndMovementDlg = NULL;
HWND g_hwndDisplayDlg = NULL;
HWND g_hwndAboutDlg = NULL;

HINSTANCE   g_hInstance = NULL;

void EnableApplyButton( HWND hwndPage)
{
    //only enable apply now if Neko is running
    if( g_hwndNeko ) PropSheet_Changed( GetParent( hwndPage ), hwndPage );
}

void UpdateNekoSettings()
{
    DWORD dwSpeed, dwSensitivity;
    BOOL bTaskbar;
    char lpstrLibrary[MAX_PATH] = "";


    //extract settings from all dialogs...
    //movement
    dwSpeed = SendDlgItemMessage( g_hwndMovementDlg, IDC_SPEEDSLIDER, TBM_GETPOS, 0, 0 );
    dwSensitivity = SendDlgItemMessage( g_hwndMovementDlg, IDC_SENSESLIDER, TBM_GETPOS, 0, 0 );
    dwSensitivity = MAX_SENSE - dwSensitivity;

    //display
    bTaskbar = ( IsDlgButtonChecked( g_hwndDisplayDlg, IDC_TASKBAR ) == 1 );
    GetDlgItemText( g_hwndDisplayDlg, IDC_IMAGELIB, lpstrLibrary, MAX_PATH-1 );

    //update settings in the registry
    CNekoSettings * NekoSettings = new CNekoSettings( szNekoRegKey );

    NekoSettings->SetInt( szNekoSpeedKey, dwSpeed );
    NekoSettings->SetInt( szNekoSenseKey, dwSensitivity );
    NekoSettings->SetBool( szNekoTaskbarKey, bTaskbar );
    NekoSettings->SetString( szNekoLibraryKey, lpstrLibrary );

    delete NekoSettings;

    //notify Neko
    if( g_hwndNeko ) SendMessage( g_hwndNeko, MY_UPDATENEKO, 0, 0 );
}

/***********************************************************/
BOOL CALLBACK MovementDlgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) 
{
    switch( uMsg )
    {
        case WM_INITDIALOG:
        {
            DWORD dwSpeed = 16, dwSensitivity = 6;

            //set slider max/min values
            SendDlgItemMessage( hWnd, IDC_SPEEDSLIDER, TBM_SETRANGE, FALSE, MAKELONG( MIN_SPEED, MAX_SPEED ) );
            SendDlgItemMessage( hWnd, IDC_SENSESLIDER, TBM_SETRANGE, FALSE, MAKELONG( MIN_SENSE, MAX_SENSE ) );

            //extract settings from the registry
            CNekoSettings * NekoSettings = new CNekoSettings( szNekoRegKey );
            NekoSettings->GetInt( szNekoSpeedKey, &dwSpeed );
            NekoSettings->GetInt( szNekoSenseKey, &dwSensitivity );
            delete NekoSettings;

            //update dialog with settings
            SendDlgItemMessage( hWnd, IDC_SPEEDSLIDER, TBM_SETPOS, TRUE, dwSpeed );
            SendDlgItemMessage( hWnd, IDC_SENSESLIDER, TBM_SETPOS, TRUE, dwSensitivity);

            //store the dialog handle
            g_hwndMovementDlg = hWnd;

            //set the default item
            SetFocus( GetDlgItem( hWnd, IDC_SPEEDSLIDER ) );
            break;
        }

        case WM_HSCROLL:
            EnableApplyButton( hWnd );
            break;

        case WM_NOTIFY:
            switch( ((NMHDR*)lParam)->code )
            {
                case PSN_APPLY:
                    UpdateNekoSettings();
                    break;

                case PSN_SETACTIVE:
                    SetFocus( GetDlgItem( hWnd, IDC_SPEEDSLIDER ) );
                    break;
            }
            break;

        case WM_COMMAND:
            EnableApplyButton( hWnd );
            break;
            
        default:
            return FALSE;
            break;
    }
    return TRUE;
}

BOOL CALLBACK DisplayDlgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) 
{
    static HICON s_hiconPreview = NULL;

    switch( uMsg )
    {
        case WM_INITDIALOG:
        {
            BOOL bTaskbar = FALSE;
            char lpstrLibrary[MAX_PATH] = "";

            //get settings from the registry
            CNekoSettings * NekoSettings = new CNekoSettings( szNekoRegKey );
            NekoSettings->GetBool( szNekoTaskbarKey, &bTaskbar );
            NekoSettings->GetString( szNekoLibraryKey, lpstrLibrary, MAX_PATH-1 );
            delete NekoSettings;

            //update dialog with settings
            CheckDlgButton( hWnd, IDC_TASKBAR, bTaskbar ? 1 : 0 );
            SetDlgItemText( hWnd, IDC_IMAGELIB, lpstrLibrary );

            if( strcmp( lpstrLibrary, "" ) != 0 )
            {
                s_hiconPreview = ExtractIcon( g_hInstance, lpstrLibrary, 0 );
                SendDlgItemMessage( hWnd, IDC_PREVIEW, STM_SETIMAGE, IMAGE_ICON, (LPARAM)s_hiconPreview );
            }

            //store the dialog handle
            g_hwndDisplayDlg = hWnd;

            //set the default item
            SetFocus( GetDlgItem( hWnd, IDC_CHANGE ) );
            break;
        }

        case WM_NOTIFY:
            switch( ((NMHDR*)lParam)->code)
            {
                case PSN_APPLY:
                    //(done in other page)UpdateNekoSettings();
                    break;

                case PSN_SETACTIVE:
                    SetFocus( GetDlgItem( hWnd, IDC_CHANGE ) );
                    break;
            }
            break;

        case WM_COMMAND:
        
            switch( LOWORD(wParam) )
            {
                case IDC_DEFAULT:
                    SetDlgItemText( hWnd, IDC_IMAGELIB, "" );
                    EnableApplyButton( hWnd );

                    if( s_hiconPreview ) DestroyIcon( s_hiconPreview );
                    s_hiconPreview = NULL;
                    s_hiconPreview = LoadIcon( g_hInstance, MAKEINTRESOURCE( IDI_DEFAULT ) );
                    SendDlgItemMessage( hWnd, IDC_PREVIEW, STM_SETIMAGE, IMAGE_ICON, (LPARAM)s_hiconPreview );
                    break;

                case IDC_CHANGE:
                {
                    OPENFILENAME ofn;
                    char lpstrLibrary[MAX_PATH];
                    LPCTSTR lpstrFileMasks = "Icon Libraries\0*.DLL;*.EXE;*.ICL\0All Files (*.*)\0*.*\0\0";

                    //get the current file
                    GetDlgItemText( hWnd, IDC_IMAGELIB, lpstrLibrary, MAX_PATH-1 );
                    
                    ofn.lStructSize = sizeof(OPENFILENAME);
                    ofn.hwndOwner = hWnd;
                    ofn.hInstance = NULL;
                    ofn.lpstrFilter = lpstrFileMasks;
                    ofn.lpstrCustomFilter = NULL;
                    ofn.nFilterIndex = 1;
                    ofn.lpstrFile = lpstrLibrary;
                    ofn.nMaxFile = MAX_PATH-1;
                    ofn.lpstrFileTitle = NULL;
                    ofn.nMaxFileTitle = 0;
                    ofn.lpstrInitialDir = NULL;
                    ofn.lpstrTitle = "Select Image Library";
                    ofn.Flags = OFN_EXPLORER|OFN_HIDEREADONLY|OFN_PATHMUSTEXIST;
                    ofn.lpstrDefExt = "ICL";//heh heh - old version forgot this (duh)
                    if( GetOpenFileName( &ofn ) )
                    {
                        //check to make sure it's got enough icons in...
                        if( (UINT)ExtractIcon( g_hInstance, ofn.lpstrFile, (UINT)-1 ) < 32 )
                            MessageBox( hWnd, "That file does not have enough icons in it - it must have at least 32", szWindowTitle, MB_ICONEXCLAMATION );
                        else
                        {
                            SetDlgItemText( hWnd, IDC_IMAGELIB, ofn.lpstrFile );
                            EnableApplyButton( hWnd );

                            //show the icon in the preview
                            if( s_hiconPreview ) DestroyIcon( s_hiconPreview );
                            s_hiconPreview = ExtractIcon( g_hInstance, lpstrLibrary, 0 );
                            SendDlgItemMessage( hWnd, IDC_PREVIEW, STM_SETIMAGE, IMAGE_ICON, (LPARAM)s_hiconPreview );
                        }
                    }
                    break;
                }

                default:
                    EnableApplyButton( hWnd );
                    break;
            }
            break;

        default:
            return FALSE;
            break;
    }
    return TRUE;
}

BOOL CALLBACK AboutDlgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) 
{
    switch( uMsg )
    {
        case WM_INITDIALOG:
        {
            //store the window handle
            g_hwndAboutDlg = hWnd;

            //set the default item
            SetFocus( GetDlgItem( hWnd, IDC_WEBPAGE ) );
            break;
        }

        case WM_NOTIFY:
            switch( ((NMHDR*)lParam)->code)
            {
                case PSN_APPLY:
                    //(done in other page)UpdateNekoSettings();
                    break;

                case PSN_SETACTIVE:
                    SetFocus( GetDlgItem( hWnd, IDC_WEBPAGE ) );
                    break;
            }
            break;

        case WM_COMMAND:
        
            switch( LOWORD(wParam) )
            {
                case IDC_WEBPAGE:
                    if( (int)ShellExecute( NULL, "open", "http://www.uea.ac.uk/~u9530134", 0, "", SW_SHOWNORMAL ) <= 32 )
                        MessageBox( hWnd, "It was not possible to open the requested URL", szWindowTitle, MB_ICONEXCLAMATION );
                    break;

                case IDC_EMAIL:
                    if( (int)ShellExecute( NULL, "open", "mailto:d.harvey@uea.ac.uk", 0, "", SW_SHOWNORMAL ) <= 32 )
                        MessageBox( hWnd, "It was not possible to open the mailing URL", szWindowTitle, MB_ICONEXCLAMATION );
                    break;

            }
            break;

        default:
            return FALSE;
            break;
    }
    return TRUE;
}


void DisplayDialogBox()
{
    PROPSHEETPAGE psp[3];
    PROPSHEETHEADER psh;

    psp[0].dwSize = sizeof(PROPSHEETPAGE);
    psp[0].dwFlags = PSP_USEICONID;
    psp[0].hInstance = g_hInstance;
    psp[0].pszTemplate = MAKEINTRESOURCE( IDD_MOVEMENT );
    psp[0].pfnDlgProc = (DLGPROC)MovementDlgProc;
    psp[0].lParam = 0;
    psp[0].pfnCallback = NULL;
    psp[0].pszIcon = MAKEINTRESOURCE( IDI_MOVEMENTICON );

    psp[1].dwSize = sizeof(PROPSHEETPAGE);
    psp[1].dwFlags = PSP_USEICONID;
    psp[1].hInstance = g_hInstance;
    psp[1].pszTemplate = MAKEINTRESOURCE( IDD_DISPLAY );
    psp[1].pfnDlgProc = (DLGPROC)DisplayDlgProc;
    psp[1].lParam = 0;
    psp[1].pfnCallback = NULL;
    psp[1].pszIcon = MAKEINTRESOURCE( IDI_DISPLAYICON );

    psp[2].dwSize = sizeof(PROPSHEETPAGE);
    psp[2].dwFlags = PSP_USEICONID;
    psp[2].hInstance = g_hInstance;
    psp[2].pszTemplate = MAKEINTRESOURCE( IDD_ABOUT );
    psp[2].pfnDlgProc = (DLGPROC)AboutDlgProc;
    psp[2].lParam = 0;
    psp[2].pfnCallback = NULL;
    psp[2].pszIcon = MAKEINTRESOURCE( IDI_ABOUTICON );


    psh.dwSize = sizeof(PROPSHEETHEADER);
    psh.dwFlags = PSH_PROPSHEETPAGE|PSH_USEICONID| (g_hwndNeko ? 0 : PSH_NOAPPLYNOW);
    psh.hInstance = g_hInstance;
    psh.hwndParent = NULL;
    psh.pszCaption = szWindowTitle;
    psh.nPages = sizeof(psp)/sizeof(PROPSHEETPAGE);
    psh.nStartPage = 0;
    psh.ppsp = (LPCPROPSHEETPAGE)&psp;
    psh.pszIcon = MAKEINTRESOURCE( IDI_APPICON );

    PropertySheet( &psh );
    PostQuitMessage( 0 );
}

int CALLBACK WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow )
{
    MSG msg;

    //ensure the common control library is loaded
    InitCommonControls();

    //store the instance handle and find the Neko window
    g_hInstance = hInstance;
    g_hwndNeko = FindWindow( szNekoClassName, szNekoWindowTitle );

    //initialise configuration dialog box
    DisplayDialogBox();

    while( GetMessage( &msg, NULL, 0, 0 ) )
    {
        TranslateMessage( &msg );
        DispatchMessage( &msg );
    }

    return ( msg.wParam );
}


