/************************************
 
        Neko for Windows 95   

 *************************************/

#include <stdlib.h>
#include <Windows.h> 
#include <commctrl.h>
#include "Neko95.hpp"
#include "Tray.hpp"
#include "NekoSettings.hpp"
#include "desktop.h"
#include "NekoCommon.h"
#include "resource.h"

//global variables
HWND      g_hwndMain = NULL;
HINSTANCE g_hInstance = NULL;
CNeko *   Neko = NULL;

//global settings
BOOL g_bTaskbar = TRUE;


//global helper class
CTray * Tray = NULL;


void ExecuteConfigProcess()
{
    if( (int)ShellExecute( NULL, "open", "NekoCFG.EXE", 0, "", SW_SHOWNORMAL ) <= 32 )
        MessageBox( g_hwndMain, "Neko was unable to start the configuration program\n'NekoCFG.EXE'\n\nMake sure that this file is in the same folder as Neko95.", "Configure Neko", MB_ICONEXCLAMATION );
}

LRESULT CALLBACK WndProc( HWND hWnd, UINT message, WPARAM uParam, LPARAM lParam )
{
    switch( message )
    {
        case WM_TIMER:
        {
            POINT mouse;
            GetCursorPos( &mouse );

            Neko->RunTowards( mouse.x, mouse.y );
            break;
        }

        case WM_DESTROY:
            PostQuitMessage( 0 );
            break;

        case WM_COMMAND:
            switch(LOWORD(uParam))
            {
                case ID_SETTINGS:
                    ExecuteConfigProcess();
                    break;

                case ID_EXIT: 
                    PostQuitMessage( 0 ); 
                    break;
            }
            break;

        case WM_WINDOWPOSCHANGING:
        case WM_DEVMODECHANGE:
        case WM_DISPLAYCHANGE:
        case WM_SETTINGCHANGE:
            // this still does not compensate for when the taskbar is moved :(
            Neko->RecalculateWindowBounds();
            break;

        case MY_UPDATENEKO:
        {
            //tell Neko to update
            Neko->LoadConfiguration();

            //get the other program settings
            CNekoSettings * NekoSettings = new CNekoSettings( szNekoRegKey );
            NekoSettings->GetBool( szNekoTaskbarKey, &g_bTaskbar );
            delete NekoSettings;

            //update the system tray
            if( g_bTaskbar )
            {
                if( Tray->GetCount() == 0 ) 
                {
                    Tray->AddIcon( g_hwndMain, LoadIcon( g_hInstance, MAKEINTRESOURCE( IDI_AWAKE )), 1 );
                }
            }
            else
            {
                if( Tray->GetCount() > 0 ) 
                {
                    Tray->RemoveIcon( g_hwndMain, 1 );
                }
            }

            break;
        }

        case MY_NOTIFYICON:
            switch( lParam )
            {
                case WM_LBUTTONDBLCLK:
                    ExecuteConfigProcess();
                    break;

                case WM_RBUTTONUP:
                    Tray->ShowPopupMenu( hWnd );
                    break;
            }
            break;

        default: 
            return DefWindowProc( hWnd, message, uParam, lParam );
    }

    return 0;
}



BOOL InitApplication( HINSTANCE hInstance )
{
    //store the instance handle
    g_hInstance = hInstance;

    //create the (hidden) window
    WNDCLASS wc;
    wc.style = 0;
    wc.lpfnWndProc = (WNDPROC)WndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = NULL;
    wc.hCursor = NULL;
    wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
    wc.lpszMenuName = NULL;
    wc.lpszClassName = szNekoClassName;

    if( RegisterClass(&wc) )
    {
        //create the window
        g_hwndMain = CreateWindow( szNekoClassName, szNekoWindowTitle, WS_OVERLAPPEDWINDOW, 0, 0, 0, 0, NULL, NULL, hInstance, NULL);

        //resize it to the size of the desktop
        RECT rcWindow;

        GetClientRect( Neko->GetDefaultWindow(), &rcWindow );
        MoveWindow( g_hwndMain, 0, 0, rcWindow.right, rcWindow.bottom, FALSE );

        //***DEBUG***
        //ShowWindow( g_hwndMain, SW_SHOW );
        //UpdateWindow( g_hwndMain );
        //***********

    }

    return ( g_hwndMain!=NULL );
}

int CALLBACK WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow )
{
    MSG msg;

    //create a Neko
    Neko = new CNeko( g_hInstance );

    //if Neko is already loaded, unload this one and the other one
    if( Neko->IsPreviousInstance() ) 
    {
        Neko->UnloadPreviousInstance();
        delete Neko;
        return FALSE;
    }


    //initialise the program
    if( !hPrevInstance ) 
        if( !InitApplication( hInstance ) ) 
        {
            delete Neko;
            return FALSE;
        }


    //initialise random number generator
    srand( GetTickCount() );

    //load global settings
    CNekoSettings * NekoSettings = new CNekoSettings( szNekoRegKey );
    NekoSettings->GetBool( szNekoTaskbarKey, &g_bTaskbar );
    delete NekoSettings;

    //create the tray object
    Tray = new CTray( hInstance );

    //add a taskbar icon, if needed
    if( g_bTaskbar ) Tray->AddIcon( g_hwndMain, LoadIcon( hInstance, MAKEINTRESOURCE( IDI_AWAKE )), 1 );

    //set a timer on the main window
    SetTimer( g_hwndMain, ID_TIMER_NEKOUPDATE, 125, NULL );

    while( GetMessage( &msg, NULL, 0, 0 ) )
    {
        TranslateMessage( &msg );
        DispatchMessage( &msg );
    }

    //remove the neko
    delete Neko;

    //remove the taskbar icon
    if( g_bTaskbar ) Tray->RemoveIcon( g_hwndMain, 1 );

    //remove the tray
    delete Tray;

    return (msg.wParam);
}
